* Re: git-svn fails to fetch repository
From: Vladimir Pouzanov @ 2009-01-13 15:03 UTC (permalink / raw)
To: git
In-Reply-To: <loom.20090113T145019-951@post.gmane.org>
Also just tried on linux box (git 1.6.0.4, perl 5.8.8). Got all the revisions
but git segfaulted at the end:
...
r21 = e839272549fd746cf45542a7aa6cb151ae3813da (trunk)
Checked out HEAD:
http://qsb-mac.googlecode.com/svn/trunk r21
Segmentation fault
^ permalink raw reply
* git-svn fails to fetch repository
From: Vladimir Pouzanov @ 2009-01-13 14:53 UTC (permalink / raw)
To: git
Hi all,
I'm trying to fetch svn repository:
git svn clone http://qsb-mac.googlecode.com/svn/trunk qsb-mac
This one fails at random time at:
Temp file with moniker ' at /opt/local/lib/perl5/site_perl/5.8.8/Git.pm
line 1011.
I know nothing about perl, so can't make anything out of that. Any hints?
Running Git 1.6.1, perl 5.8.8, OSX 10.5.6
PS: Please CC me on answer.
^ permalink raw reply
* Re: [PATCH,v2] contrib/examples/README: give an explanation of the status of these files
From: Andreas Ericsson @ 2009-01-13 13:43 UTC (permalink / raw)
To: Thomas Adam; +Cc: jidanni, gitster, git
In-Reply-To: <18071eea0901130201r41c576e6t7f8c9fda7259e9f2@mail.gmail.com>
Thomas Adam wrote:
> 2009/1/13 <jidanni@jidanni.org>:
>> @@ -0,0 +1,3 @@
>> +These are original scripted implementations, kept primarily for their
>> +reference value to any aspiring plumbing users who want to learn how
>> +pieces can be fit together.
>
> "... who want to learn how the pieces fit together" is the correct
> phrasing. The above is incorrect.
>
Actually, "can be fitted together" would be correct, as we're not
talking about a jigsaw puzzle with just one solution, but rather
a few possible ways of building something out of the lego-style
programs that make up git.
--
Andreas Ericsson andreas.ericsson@op5.se
OP5 AB www.op5.se
Tel: +46 8-230225 Fax: +46 8-230231
^ permalink raw reply
* gitignore excludes too much
From: Jan Engelhardt @ 2009-01-13 13:30 UTC (permalink / raw)
To: git
Hi,
I noticed that having "*.d" in .gitignore ignores files that would start
with a dot, such as ".main.o.d". This is against Unix shell behavior;
but maybe it's a feature rather than a bug?
In case of latter, please fix :-)
Jan
^ permalink raw reply
* Re: How to pull remote branch with specified commit id?
From: Emily Ren @ 2009-01-13 13:15 UTC (permalink / raw)
To: Andreas Ericsson; +Cc: Git Mailinglist
In-Reply-To: <496C5BE3.2040206@op5.se>
Andreas,
I tried your method, it works. Thank you very much !
Emily
On Tue, Jan 13, 2009 at 5:16 PM, Andreas Ericsson <ae@op5.se> wrote:
> Emily Ren wrote:
>>
>> Git experts,
>>
>> I want to pull remote branch with specified commit id, how to do it?
>>
>> Below command can get remote branch
>> $git pull remote refs/heads/$branch_name
>>
>> Below command doesn't work
>> $git pull remote objects/$commit_id
>>
>
> You need to fetch it first, and then merge the commit you want. The
> tools operating the fetching protocol only use refs, so if you want
> to fetch (or pull) a specific version that has neither a tag nor a
> branch head pointing to it, you'll have to write a new tool for that.
>
> The end-result of the following command will be, barring side-effects
> in the remote-tracking branches, identical to what you're trying to
> do though:
> git fetch remote && git merge $commit_id
>
> --
> Andreas Ericsson andreas.ericsson@op5.se
> OP5 AB www.op5.se
> Tel: +46 8-230225 Fax: +46 8-230231
>
^ permalink raw reply
* [PATCH/RFC v7 5/5] lstat_cache(): introduce clear_lstat_cache() function
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
In-Reply-To: <1231849748-8244-1-git-send-email-barvik@broadpark.no>
If you want to completely clear the contents of the lstat_cache(), then
call this new function.
Signed-off-by: Kjetil Barvik <barvik@broadpark.no>
---
cache.h | 1 +
symlinks.c | 7 +++++++
2 files changed, 8 insertions(+), 0 deletions(-)
diff --git a/cache.h b/cache.h
index f4452a8..468daf6 100644
--- a/cache.h
+++ b/cache.h
@@ -723,6 +723,7 @@ extern int has_symlink_leading_path(int len, const char *name);
extern int has_symlink_or_noent_leading_path(int len, const char *name);
extern int has_dirs_only_path(int len, const char *name, int prefix_len);
extern void invalidate_lstat_cache(int len, const char *name);
+extern void clear_lstat_cache(void);
extern struct alternate_object_database {
struct alternate_object_database *next;
diff --git a/symlinks.c b/symlinks.c
index 3b1c3da..7a634cb 100644
--- a/symlinks.c
+++ b/symlinks.c
@@ -183,6 +183,13 @@ void invalidate_lstat_cache(int len, const char *name)
}
}
+/* Completely clear the contents of the cache!
+ */
+void clear_lstat_cache(void)
+{
+ reset_lstat_cache(0, 0);
+}
+
#define USE_ONLY_LSTAT 0
/* Return non-zero if path 'name' has a leading symlink component.
--
1.6.0.2.GIT
^ permalink raw reply related
* [PATCH/RFC v7 4/5] lstat_cache(): introduce invalidate_lstat_cache() function
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
In-Reply-To: <1231849748-8244-1-git-send-email-barvik@broadpark.no>
In some cases it could maybe be necessary to say to the cache that
"Hey, I deleted this directory and if you currently has it inside your
cache, you should deleted it". This patch introduce a function which
support this.
Signed-off-by: Kjetil Barvik <barvik@broadpark.no>
---
cache.h | 1 +
symlinks.c | 39 +++++++++++++++++++++++++++++++--------
2 files changed, 32 insertions(+), 8 deletions(-)
diff --git a/cache.h b/cache.h
index 7c8c8e4..f4452a8 100644
--- a/cache.h
+++ b/cache.h
@@ -722,6 +722,7 @@ extern int checkout_entry(struct cache_entry *ce, const struct checkout *state,
extern int has_symlink_leading_path(int len, const char *name);
extern int has_symlink_or_noent_leading_path(int len, const char *name);
extern int has_dirs_only_path(int len, const char *name, int prefix_len);
+extern void invalidate_lstat_cache(int len, const char *name);
extern struct alternate_object_database {
struct alternate_object_database *next;
diff --git a/symlinks.c b/symlinks.c
index f9d1821..3b1c3da 100644
--- a/symlinks.c
+++ b/symlinks.c
@@ -11,23 +11,30 @@ static struct cache_def {
/* Returns the length (on a path component basis) of the longest
* common prefix match of 'name' and the cached path string.
*/
-static inline int greatest_match_lstat_cache(int len, const char *name)
+static inline int greatest_match_lstat_cache(int len, const char *name,
+ int *previous_slash)
{
- int max_len, match_len = 0, i = 0;
+ int max_len, match_len = 0, match_len_prev = 0, i = 0;
max_len = len < cache.len ? len : cache.len;
while (i < max_len && name[i] == cache.path[i]) {
- if (name[i] == '/')
+ if (name[i] == '/') {
+ match_len_prev = match_len;
match_len = i;
+ }
i++;
}
/* Is the cached path string a substring of 'name'? */
- if (i == cache.len && len > cache.len && name[cache.len] == '/')
+ if (i == cache.len && len > cache.len && name[cache.len] == '/') {
+ match_len_prev = match_len;
match_len = cache.len;
/* Is 'name' a substring of the cached path string? */
- else if ((i == len && len < cache.len && cache.path[len] == '/') ||
- (i == len && len == cache.len))
+ } else if ((i == len && len < cache.len && cache.path[len] == '/') ||
+ (i == len && len == cache.len)) {
+ match_len_prev = match_len;
match_len = len;
+ }
+ *previous_slash = match_len_prev;
return match_len;
}
@@ -62,7 +69,7 @@ static inline void reset_lstat_cache(int track_flags, int prefix_len_stat_func)
static int lstat_cache(int len, const char *name,
int track_flags, int prefix_len_stat_func)
{
- int match_len, last_slash, last_slash_dir;
+ int match_len, last_slash, last_slash_dir, previous_slash;
int match_flags, ret_flags, save_flags, max_len, ret;
struct stat st;
@@ -80,7 +87,8 @@ static int lstat_cache(int len, const char *name,
* Check to see if we have a match from the cache for
* the 2 "excluding" path types.
*/
- match_len = last_slash = greatest_match_lstat_cache(len, name);
+ match_len = last_slash =
+ greatest_match_lstat_cache(len, name, &previous_slash);
match_flags = cache.flags & track_flags & (FL_NOENT|FL_SYMLINK);
if (match_flags && match_len == cache.len)
return match_flags;
@@ -160,6 +168,21 @@ static int lstat_cache(int len, const char *name,
return ret_flags;
}
+/* Invalidate the given 'name' from the cache, if 'name' matches
+ * completely with the cache.
+ */
+void invalidate_lstat_cache(int len, const char *name)
+{
+ int match_len, previous_slash;
+
+ match_len = greatest_match_lstat_cache(len, name, &previous_slash);
+ if (match_len == len) {
+ cache.path[previous_slash] = '\0';
+ cache.len = previous_slash;
+ cache.flags = previous_slash ? FL_DIR : 0;
+ }
+}
+
#define USE_ONLY_LSTAT 0
/* Return non-zero if path 'name' has a leading symlink component.
--
1.6.0.2.GIT
^ permalink raw reply related
* [PATCH/RFC v7 3/5] lstat_cache(): introduce has_dirs_only_path() function
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
In-Reply-To: <1231849748-8244-1-git-send-email-barvik@broadpark.no>
Inside the create_directories() function inside entry.c we currently
do an stat() or lstat() call for each path component of the pathname
'path' each and every time called. For the 'git checkout' command,
this function is called on each file for which we must do an update
(ce->ce_flags & CE_UPDATE), so we get lots and lots of calls...
To fix this, we make a new wrapper to the lstat_cache() function, and
call the wrapper function instead of the calls to the stat() or the
lstat() functions. Since the paths given to the create_directories()
function, is sorted alphabetically, the new wrapper would be very
cache effective in this situation.
To support it we must update the lstat_cache() function to be able to
say that "pleace test the complete length of 'name'", and also to give
it the length of a prefix, where the cache should use the stat()
function instead of the lstat() function to test each path component.
Thanks to Junio C Hamano, Linus Torvalds and Rene Scharfe for valuable
comments to this patch!
Signed-off-by: Kjetil Barvik <barvik@broadpark.no>
---
cache.h | 1 +
entry.c | 34 ++++++++++---------------------
symlinks.c | 63 +++++++++++++++++++++++++++++++++++++++++++----------------
3 files changed, 58 insertions(+), 40 deletions(-)
diff --git a/cache.h b/cache.h
index 11181aa..7c8c8e4 100644
--- a/cache.h
+++ b/cache.h
@@ -721,6 +721,7 @@ struct checkout {
extern int checkout_entry(struct cache_entry *ce, const struct checkout *state, char *topath);
extern int has_symlink_leading_path(int len, const char *name);
extern int has_symlink_or_noent_leading_path(int len, const char *name);
+extern int has_dirs_only_path(int len, const char *name, int prefix_len);
extern struct alternate_object_database {
struct alternate_object_database *next;
diff --git a/entry.c b/entry.c
index aa2ee46..b0295bd 100644
--- a/entry.c
+++ b/entry.c
@@ -8,35 +8,23 @@ static void create_directories(const char *path, const struct checkout *state)
const char *slash = path;
while ((slash = strchr(slash+1, '/')) != NULL) {
- struct stat st;
- int stat_status;
-
len = slash - path;
memcpy(buf, path, len);
buf[len] = 0;
- if (len <= state->base_dir_len)
- /*
- * checkout-index --prefix=<dir>; <dir> is
- * allowed to be a symlink to an existing
- * directory.
- */
- stat_status = stat(buf, &st);
- else
- /*
- * if there currently is a symlink, we would
- * want to replace it with a real directory.
- */
- stat_status = lstat(buf, &st);
-
- if (!stat_status && S_ISDIR(st.st_mode))
+ /* For 'checkout-index --prefix=<dir>', <dir> is
+ * allowed to be a symlink to an existing directory,
+ * and we set 'state->base_dir_len' below, such that
+ * we test the path components of the prefix with the
+ * stat() function instead of the lstat() function.
+ */
+ if (has_dirs_only_path(len, buf, state->base_dir_len))
continue; /* ok, it is already a directory. */
- /*
- * We know stat_status == 0 means something exists
- * there and this mkdir would fail, but that is an
- * error codepath; we do not care, as we unlink and
- * mkdir again in such a case.
+ /* If this mkdir() would fail, it could be that there
+ * is already a symlink or something else exists
+ * there, therefore we then try to unlink it and try
+ * one more time to create the directory.
*/
if (mkdir(buf, 0777)) {
if (errno == EEXIST && state->force &&
--
1.6.0.2.GIT
^ permalink raw reply related
* [PATCH/RFC v7 2/5] lstat_cache(): introduce has_symlink_or_noent_leading_path() function
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
In-Reply-To: <1231849748-8244-1-git-send-email-barvik@broadpark.no>
In some cases, especially inside the unpack-trees.c file, and inside
the verify_absent() function, we can avoid some unnecessary calls to
lstat(), if the lstat_cache() function can also be told to keep track
of none existing directories.
So we update the lstat_cache() function to handle this new fact,
introduce a new wrapper function, and the result is that we save lots
of lstat() calls for a removed directory which previously contained
lots of files, when we call this new wrapper of lstat_cache() instead
of the old one.
We do similar changes inside the unlink_entry() function, since if we
can already say that the leading directory component of a pathname
does not exist, it is not necessary to try to remove a pathname below
it!
Thanks to Junio C Hamano, Linus Torvalds and Rene Scharfe for valuable
comments to this patch!
Signed-off-by: Kjetil Barvik <barvik@broadpark.no>
---
cache.h | 1 +
symlinks.c | 87 ++++++++++++++++++++++++++++++++++++--------------------
unpack-trees.c | 4 +-
3 files changed, 59 insertions(+), 33 deletions(-)
diff --git a/cache.h b/cache.h
index 231c06d..11181aa 100644
--- a/cache.h
+++ b/cache.h
@@ -720,6 +720,7 @@ struct checkout {
extern int checkout_entry(struct cache_entry *ce, const struct checkout *state, char *topath);
extern int has_symlink_leading_path(int len, const char *name);
+extern int has_symlink_or_noent_leading_path(int len, const char *name);
extern struct alternate_object_database {
struct alternate_object_database *next;
diff --git a/unpack-trees.c b/unpack-trees.c
index 54f301d..a3fd383 100644
--- a/unpack-trees.c
+++ b/unpack-trees.c
@@ -61,7 +61,7 @@ static void unlink_entry(struct cache_entry *ce)
char *cp, *prev;
char *name = ce->name;
- if (has_symlink_leading_path(ce_namelen(ce), ce->name))
+ if (has_symlink_or_noent_leading_path(ce_namelen(ce), ce->name))
return;
if (unlink(name))
return;
@@ -584,7 +584,7 @@ static int verify_absent(struct cache_entry *ce, const char *action,
if (o->index_only || o->reset || !o->update)
return 0;
- if (has_symlink_leading_path(ce_namelen(ce), ce->name))
+ if (has_symlink_or_noent_leading_path(ce_namelen(ce), ce->name))
return 0;
if (!lstat(ce->name, &st)) {
--
1.6.0.2.GIT
^ permalink raw reply related
* [PATCH/RFC v7 1/5] lstat_cache(): more cache effective symlink/directory detection
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
In-Reply-To: <1231849748-8244-1-git-send-email-barvik@broadpark.no>
Make the cache functionality more effective. Previously when A/B/C/D
was in the cache and A/B/C/E/file.c was called for, there was no match
at all from the cache. Now we use the fact that the paths "A", "A/B"
and "A/B/C" are already tested, and we only need to do an lstat() call
on "A/B/C/E".
We only cache/store the last path regardless of its type. Since the
cache functionality is always used with alphabetically sorted names
(at least it seems so for me), there is no need to store both the last
symlink-leading path and the last real-directory path. Note that if
the cache is not called with (mostly) alphabetically sorted names,
neither the old, nor this new one, would be very effective.
Previously, when symlink A/B/C/S was cached/stored in the symlink-
leading path, and A/B/C/file.c was called for, it was not easy to use
the fact that we already knew that the paths "A", "A/B" and "A/B/C"
are real directories.
Avoid copying the first path components of the name 2 zillion times
when we test new path components. Since we always cache/store the
last path, we can copy each component as we test those directly into
the cache. Previously we ended up doing a memcpy() for the full
path/name right before each lstat() call, and when updating the cache
for each time we have tested a new path component.
We also use less memory, that is, PATH_MAX bytes less memory on the
stack and PATH_MAX bytes less memory on the heap.
Thanks to Junio C Hamano, Linus Torvalds and Rene Scharfe for valuable
comments to this patch!
Signed-off-by: Kjetil Barvik <barvik@broadpark.no>
---
symlinks.c | 157 ++++++++++++++++++++++++++++++++++++++++++++---------------
1 files changed, 117 insertions(+), 40 deletions(-)
diff --git a/symlinks.c b/symlinks.c
index 5a5e781..b17e4b3 100644
--- a/symlinks.c
+++ b/symlinks.c
@@ -1,64 +1,141 @@
#include "cache.h"
-struct pathname {
- int len;
+static struct cache_def {
char path[PATH_MAX];
-};
+ int len;
+ int flags;
+} cache;
-/* Return matching pathname prefix length, or zero if not matching */
-static inline int match_pathname(int len, const char *name, struct pathname *match)
+/* Returns the length (on a path component basis) of the longest
+ * common prefix match of 'name' and the cached path string.
+ */
+static inline int greatest_match_lstat_cache(int len, const char *name)
{
- int match_len = match->len;
- return (len > match_len &&
- name[match_len] == '/' &&
- !memcmp(name, match->path, match_len)) ? match_len : 0;
+ int max_len, match_len = 0, i = 0;
+
+ max_len = len < cache.len ? len : cache.len;
+ while (i < max_len && name[i] == cache.path[i]) {
+ if (name[i] == '/')
+ match_len = i;
+ i++;
+ }
+ /* Is the cached path string a substring of 'name'? */
+ if (i == cache.len && len > cache.len && name[cache.len] == '/')
+ match_len = cache.len;
+ /* Is 'name' a substring of the cached path string? */
+ else if ((i == len && len < cache.len && cache.path[len] == '/') ||
+ (i == len && len == cache.len))
+ match_len = len;
+ return match_len;
}
-static inline void set_pathname(int len, const char *name, struct pathname *match)
+static inline void reset_lstat_cache(void)
{
- if (len < PATH_MAX) {
- match->len = len;
- memcpy(match->path, name, len);
- match->path[len] = 0;
- }
+ cache.path[0] = '\0';
+ cache.len = 0;
+ cache.flags = 0;
}
-int has_symlink_leading_path(int len, const char *name)
+#define FL_DIR (1 << 0)
+#define FL_SYMLINK (1 << 1)
+#define FL_LSTATERR (1 << 2)
+#define FL_ERR (1 << 3)
+
+/*
+ * Check if name 'name' of length 'len' has a symlink leading
+ * component, or if the directory exists and is real.
+ *
+ * To speed up the check, some information is allowed to be cached.
+ * This is can be indicated by the 'track_flags' argument.
+ */
+static int lstat_cache(int len, const char *name,
+ int track_flags)
{
- static struct pathname link, nonlink;
- char path[PATH_MAX];
+ int match_len, last_slash, last_slash_dir;
+ int match_flags, ret_flags, save_flags, max_len;
struct stat st;
- char *sp;
- int known_dir;
/*
- * See if the last known symlink cache matches.
+ * Check to see if we have a match from the cache for the
+ * symlink path type.
*/
- if (match_pathname(len, name, &link))
- return 1;
-
+ match_len = last_slash = greatest_match_lstat_cache(len, name);
+ match_flags = cache.flags & track_flags & FL_SYMLINK;
+ if (match_flags && match_len == cache.len)
+ return match_flags;
/*
- * Get rid of the last known directory part
+ * If 'name' is a substring of the cache on a path component
+ * basis, and a directory is cached, we can return
+ * immediately.
*/
- known_dir = match_pathname(len, name, &nonlink);
+ match_flags = cache.flags & track_flags & FL_DIR;
+ if (match_flags && match_len == len)
+ return match_flags;
- while ((sp = strchr(name + known_dir + 1, '/')) != NULL) {
- int thislen = sp - name ;
- memcpy(path, name, thislen);
- path[thislen] = 0;
+ /* Okay, no match from the cache so far, so now we have to
+ * check the rest of the path components.
+ */
+ ret_flags = FL_DIR;
+ last_slash_dir = last_slash;
+ max_len = len < PATH_MAX ? len : PATH_MAX;
+ while (match_len < max_len) {
+ do {
+ cache.path[match_len] = name[match_len];
+ match_len++;
+ } while (match_len < max_len && name[match_len] != '/');
+ if (match_len >= max_len)
+ break;
+ last_slash = match_len;
+ cache.path[last_slash] = '\0';
- if (lstat(path, &st))
- return 0;
- if (S_ISDIR(st.st_mode)) {
- set_pathname(thislen, path, &nonlink);
- known_dir = thislen;
+ if (lstat(cache.path, &st)) {
+ ret_flags = FL_LSTATERR;
+ } else if (S_ISDIR(st.st_mode)) {
+ last_slash_dir = last_slash;
continue;
- }
- if (S_ISLNK(st.st_mode)) {
- set_pathname(thislen, path, &link);
- return 1;
+ } else if (S_ISLNK(st.st_mode)) {
+ ret_flags = FL_SYMLINK;
+ } else {
+ ret_flags = FL_ERR;
}
break;
}
- return 0;
+
+ /* At the end update the cache. Note that max 2 different
+ * path types, FL_SYMLINK and FL_DIR, can be cached for the
+ * moment!
+ */
+ save_flags = ret_flags & track_flags & FL_SYMLINK;
+ if (save_flags && last_slash > 0 && last_slash < PATH_MAX) {
+ cache.path[last_slash] = '\0';
+ cache.len = last_slash;
+ cache.flags = save_flags;
+ } else if (track_flags & FL_DIR &&
+ last_slash_dir > 0 && last_slash_dir < PATH_MAX) {
+ /* We have a separate test for the directory case,
+ * since it could be that we have found a symlink and
+ * the track_flags says that we can not cache this
+ * fact, so the cache would then have been left empty
+ * in this case.
+ *
+ * But, if we is allowed to track real directories, we
+ * can still cache the path components before the last
+ * one (the found symlink component).
+ */
+ cache.path[last_slash_dir] = '\0';
+ cache.len = last_slash_dir;
+ cache.flags = FL_DIR;
+ } else {
+ reset_lstat_cache();
+ }
+ return ret_flags;
+}
+
+/* Return non-zero if path 'name' has a leading symlink component.
+ */
+int has_symlink_leading_path(int len, const char *name)
+{
+ return lstat_cache(len, name,
+ FL_SYMLINK|FL_DIR) &
+ FL_SYMLINK;
}
--
1.6.0.2.GIT
^ permalink raw reply related
* [PATCH/RFC v7 0/5] git checkout: optimise away lots of lstat() calls
From: Kjetil Barvik @ 2009-01-13 12:29 UTC (permalink / raw)
To: git; +Cc: Junio C Hamano, Pete Harlan, Linus Torvalds, Kjetil Barvik
Changes since version 6:
a) The patch-split was not correct for patch 3/3. In v6 it could look
like the lines:
- char path[PATH_MAX];
+ char path[PATH_MAX + 1];
was a random bugfix. In this version for patch 3/5 we also change
"&& last_slash < PATH_MAX" into "&& last_slash <= PATH_MAX" (and
similar for the else-if part). This could also make the intro-
duction of the 'FL_FULLPATH' flag more readable for this patch.
b) Cleanup and added 3 comments to 'greatest_match_lstat_cache()'
c) Introduction of the 'invalidate_lstat_cache()' function. How does
the interface look? good? bad? Does the function do what people
expect it to do?
d) Reintroduction of the 'clear_lstat_cache()' function.
Junio, I hope it is possible to use patches 1/5, 2/5 and 3/5 from this
version instead of 1/3, 2/3 and 3/3 from version 6, for the possible
future in origin/pu? See also a) above. Thanks in advance!
In general, are we allowed to redesign the patch-series while the
patches is inside origin/pu?
Kjetil Barvik (5):
lstat_cache(): more cache effective symlink/directory detection
lstat_cache(): introduce has_symlink_or_noent_leading_path() function
lstat_cache(): introduce has_dirs_only_path() function
lstat_cache(): introduce invalidate_lstat_cache() function
lstat_cache(): introduce clear_lstat_cache() function
cache.h | 4 +
entry.c | 34 +++-----
symlinks.c | 249 ++++++++++++++++++++++++++++++++++++++++++++++----------
unpack-trees.c | 4 +-
4 files changed, 222 insertions(+), 69 deletions(-)
^ permalink raw reply
* Re: how to combine 2 commits?
From: Sitaram Chamarty @ 2009-01-13 11:32 UTC (permalink / raw)
To: git
In-Reply-To: <vpqiqojpuqn.fsf@bauges.imag.fr>
On 2009-01-13, Matthieu Moy <Matthieu.Moy@imag.fr> wrote:
> Sitaram Chamarty <sitaramc@gmail.com> writes:
>
>> I like using "git rebase -i HEAD~5" (if I want to rebase the
>> last 5 commits). In the editor that pops up, I reorder the
>> ones that I know should be together, and on each set to be
>> squashed, I change the "pick" to "s" (for squash) on all but
>> the first one. Save the file and it's all done.
>
> And right after, you probably want to do stg like
>
> git diff master@{1} master
>
> to see whether you actually changed the result by reordering your
> patches.
I used to do that, but now I've stopped. If it doesn't
conflict, I know it's good. I go straight to the equivalent
of make && make test.
I must mention though, that I don't reorder the relative
sequences of commits affecting the same file/set of files.
That might have something to do with my confidence :-)
^ permalink raw reply
* Re: "git diff --stat" doesn't show added empty file
From: Johannes Schindelin @ 2009-01-13 11:27 UTC (permalink / raw)
To: Michael J Gruber; +Cc: Ping Yin, Git Mailing List
In-Reply-To: <496B64BA.7000408@drmicha.warpmail.net>
Hi,
On Mon, 12 Jan 2009, Michael J Gruber wrote:
> Johannes Schindelin venit, vidit, dixit 01/12/09 13:19:
>
> > On Mon, 12 Jan 2009, Ping Yin wrote:
> >
> >> $ git --version
> >> git version 1.6.1.9.g97c34
> >> $ mkdir test && cd test && git init && git commit --allow-empty -m
> >> "Initial commit"
> >> $ touch .gitignore && git add .gitignore && git commit -m "Add empty .gitignore"
> >> $ git diff --stat HEAD^..
> >> 0 files changed, 0 insertions(+), 0 deletions(-)
> >>
> >> May be the following is better?
> >>
> >> .gitignore | 0 +
> >> 1 files changed, 0 insertions(+), 0 deletions(-)
> >
> > Have fun coding that.
>
> Removing 5 lines from diff.c does the job,
I can only think that you mean these lines:
else if (!data->files[i]->is_renamed &&
(added + deleted == 0)) {
total_files--;
continue;
}
However, they are also present in the initial stat code in d75f7952:
} else if (added + deleted == 0) {
total_files--;
continue;
}
Now, the fun part is finding out why this code is there. Are there
unintended side effects? If so, are they still there? Or was it
introduced due to an explicit request back then? Was it just sloppy?
Removing the 5 lines is not all you have to do, by _far_.
Lots of fun,
Dscho
^ permalink raw reply
* Re: how to combine 2 commits?
From: Stephen R. van den Berg @ 2009-01-13 11:20 UTC (permalink / raw)
To: git
In-Reply-To: <20090113075121.GA6923@b2j>
bill lam wrote:
>BTW, what is the norm of reply in this list, should it be reply to
>list only, or reply to group (list plus all cc)?
This list prefers list plus all cc.
--
Sincerely,
Stephen R. van den Berg.
"I have a *cunning* plan!"
^ permalink raw reply
* Re: git/webdav is refusing to authenticate properly.
From: Johannes Schindelin @ 2009-01-13 11:11 UTC (permalink / raw)
To: Boyd Stephen Smith Jr.; +Cc: Peter Spierenburg, git
In-Reply-To: <200901121953.16183.bss@iguanasuicide.net>
Hi,
On Mon, 12 Jan 2009, Boyd Stephen Smith Jr. wrote:
> On Monday 12 January 2009, Peter Spierenburg
> <ionlyusethisaddressforlists@gmail.com> wrote
>
> >C'mon, leave my password 'in-the-clear', in a text file on a networked
> >box? That is the kind of prank a fourth-year student would try to pull
> >on a freshman.
> >
> >How do I really do it?
>
> AFAIK, that's the only way for now.
Indeed it is.
> Personally, I'd welcome a patch that allowed fetch/push to prompt the
> user for a password, but I'm not holding my breath.
I guess everybody who wants that feature does exactly the same as you.
Oh, and FWIW: trying to ridicule the way Git does it completely misses the
fact that you are using such a ridiculously weak authentication scheme
anyway that you could just as well open the window and shout your password
out loud.
Ciao,
Dscho
P.S.: real security would require you to have public/private key based
client authentication. But that would leave you vulnerable, too, as you
are apparently not on a trustable machine.
P.P.S.: I have access to such a machine, too, but I never push _from_
there. That is relatively safe, as only those can access my private keys
(and my $HOME/.netrc, BTW, as you seemed to have missed .netrc's 0600
mode) can impersonate me without hassle using sudo anyway.
^ permalink raw reply
* [TopGit PATCH] depend: don't check for tg branch
From: Thomas Rast @ 2009-01-13 11:05 UTC (permalink / raw)
To: git; +Cc: martin f. krafft
Remove the check in 'tg depend add <branch>' that <branch> is
topgit-tracked, as it seems inconsistent with the use of 'tg create
<branch> <dependencies>', where the <dependencies> can be arbitrary.
This is useful to, e.g., promote a git.git topic branch from
master-based to next-based.
Signed-off-by: Thomas Rast <trast@student.ethz.ch>
---
tg-depend.sh | 2 --
1 files changed, 0 insertions(+), 2 deletions(-)
diff --git a/tg-depend.sh b/tg-depend.sh
index ef5f94f..16b7751 100644
--- a/tg-depend.sh
+++ b/tg-depend.sh
@@ -37,8 +37,6 @@ done
[ -n "$name" ] || die "no branch name specified"
branchrev="$(git rev-parse --verify "$name" 2>/dev/null)" ||
die "invalid branch name: $name"
-baserev="$(git rev-parse --verify "refs/top-bases/$name" 2>/dev/null)" ||
- die "not a TopGit topic branch: $name"
## Record new dependency
--
tg: (3c70506..) t/depend-no-base-check (depends on: master)
^ permalink raw reply related
* Re: git-patches list?
From: Johannes Schindelin @ 2009-01-13 11:02 UTC (permalink / raw)
To: Andreas Ericsson; +Cc: Junio C Hamano, Akira Kitada, git
In-Reply-To: <496C502A.3070908@op5.se>
Hi,
On Tue, 13 Jan 2009, Andreas Ericsson wrote:
> Junio C Hamano wrote:
> > Akira Kitada <akitada@gmail.com> writes:
> >
> > > Can I propose having another mailing list for posting patches to avoid
> > > daily mail flood to this list?
> > >
> > > Yes, I can filter out the emails but still...
> >
> > This list has always been the only place where git development happens.
> > It would make the development very awkward to set up another list only for
> > patches, forbid patches to be sent to anywhere but that new list, but
> > still discuss the patches on this list.
> >
> > It does not make much sense to me.
> >
>
> I agree.
>
> If any new list is to be introduced, it would only make sense to make it
> a read-only "what's new" list, for the various people that want to know
> what might break/be fixed in new versions but really don't care about the
> discussion that lead to that decision.
If that's what you want, I suggest having a look at
http://gitrss.q42.co.uk/
I am sure that Julian can be convinced to filter "[PATCH", too.
Ciao,
Dscho
^ permalink raw reply
* Re: [PATCH,v2] contrib/examples/README: give an explanation of the status of these files
From: Thomas Adam @ 2009-01-13 10:01 UTC (permalink / raw)
To: jidanni; +Cc: gitster, git
In-Reply-To: <877i50ovdd.fsf_-_@jidanni.org>
2009/1/13 <jidanni@jidanni.org>:
> @@ -0,0 +1,3 @@
> +These are original scripted implementations, kept primarily for their
> +reference value to any aspiring plumbing users who want to learn how
> +pieces can be fit together.
"... who want to learn how the pieces fit together" is the correct
phrasing. The above is incorrect.
-- Thomas Adam
^ permalink raw reply
* Re: How to pull remote branch with specified commit id?
From: Johannes Sixt @ 2009-01-13 9:43 UTC (permalink / raw)
To: Emily Ren; +Cc: Git Mailinglist
In-Reply-To: <856bfe0e0901130108q3af1345cy31751dd09e030c96@mail.gmail.com>
Emily Ren schrieb:
> Git experts,
>
> I want to pull remote branch with specified commit id, how to do it?
>
> Below command can get remote branch
> $git pull remote refs/heads/$branch_name
>
> Below command doesn't work
> $git pull remote objects/$commit_id
You can't, and that is so by design.
Consider this: You accidentally push a branch with confidential data to a
public repository. You notice it early, and quickly delete the branch
using 'git push the-repo :refs/heads/that-branch'. At this time the
objects with the confidential data are still lingering in the public
repository. But with the current behavior noone can access them even if
the SHA1 happens to be known.
-- Hannes
^ permalink raw reply
* Re: [BUG PATCH RFC] mailinfo: correctly handle multiline 'Subject:' header
From: Kirill Smelkov @ 2009-01-13 9:39 UTC (permalink / raw)
To: Junio C Hamano; +Cc: Alexander Potashev, git
In-Reply-To: <7v63kkgl5b.fsf@gitster.siamese.dyndns.org>
On Mon, Jan 12, 2009 at 03:27:44PM -0800, Junio C Hamano wrote:
> Kirill Smelkov <kirr@landau.phys.spbu.ru> writes:
>
> > diff --git a/builtin-mailinfo.c b/builtin-mailinfo.c
> > index f7c8c08..6d72c1b 100644
> > --- a/builtin-mailinfo.c
> > +++ b/builtin-mailinfo.c
> > @@ -860,6 +860,7 @@ static void handle_info(void)
> > }
> > output_header_lines(fout, "Subject", hdr);
> > } else if (!memcmp(header[i], "From", 4)) {
> > + cleanup_space(hdr);
> > handle_from(hdr);
> > fprintf(fout, "Author: %s\n", name.buf);
> > fprintf(fout, "Email: %s\n", email.buf);
> > diff --git a/t/t5100/sample.mbox b/t/t5100/sample.mbox
> > index 4bf7947..d465685 100644
> > --- a/t/t5100/sample.mbox
> > +++ b/t/t5100/sample.mbox
> > @@ -2,7 +2,10 @@
> >
> >
> > From nobody Mon Sep 17 00:00:00 2001
> > -From: A U Thor <a.u.thor@example.com>
> > +From: A
> > + U
> > + Thor
> > + <a.u.thor@example.com>
> > Date: Fri, 9 Jun 2006 00:44:16 -0700
> > Subject: [PATCH] a commit.
>
> I think this is a reasonable change.
Thanks.
> But doesn't this
>
> > From nobody Mon Sep 17 00:00:00 2001
> > -From: A
> > +From: A (zzz)
> > U
> > Thor
> > - <a.u.thor@example.com>
> > + <a.u.thor@example.com> (Comment)
>
> regress for people who spell their names like this?
>
> From: john.doe@email.xz (John Doe)
I think everything is ok:
There is an explicit handler for such emails before my comments removal
in builtin-mailinfo.c:
/* The remainder is name. It could be "John Doe <john.doe@xz>"
* or "john.doe@xz (John Doe)", but we have removed the
* email part, so trim from both ends, possibly removing
* the () pair at the end.
*/
strbuf_trim(&f);
if (f.buf[0] == '(' && f.len && f.buf[f.len - 1] == ')') {
strbuf_remove(&f, 0, 1);
strbuf_setlen(&f, f.len - 1);
}
http://repo.or.cz/w/git.git?a=blob;f=builtin-mailinfo.c;h=f7c8c08b320c99d8bf96443ae57aa33c1de7e8c0;hb=HEAD#l112
And only a test for this is missing
From 77316ad6db2c3b0f4be238c4ba855b2f785b50d6 Mon Sep 17 00:00:00 2001
From: Kirill Smelkov <kirr@mns.spb.ru>
Date: Tue, 13 Jan 2009 12:33:48 +0300
Subject: [PATCH] mailinfo: add explicit test for mails like '<a.u.thor@example.com> (A U Thor)'
Signed-off-by: Kirill Smelkov <kirr@mns.spb.ru>
---
t/t5100-mailinfo.sh | 2 +-
t/t5100/info0013 | 5 +++++
t/t5100/sample.mbox | 5 +++++
3 files changed, 11 insertions(+), 1 deletions(-)
create mode 100644 t/t5100/info0013
create mode 100644 t/t5100/msg0013
create mode 100644 t/t5100/patch0013
diff --git a/t/t5100-mailinfo.sh b/t/t5100-mailinfo.sh
index 625c204..e70ea94 100755
--- a/t/t5100-mailinfo.sh
+++ b/t/t5100-mailinfo.sh
@@ -11,7 +11,7 @@ test_expect_success 'split sample box' \
'git mailsplit -o. "$TEST_DIRECTORY"/t5100/sample.mbox >last &&
last=`cat last` &&
echo total is $last &&
- test `cat last` = 12'
+ test `cat last` = 13'
for mail in `echo 00*`
do
diff --git a/t/t5100/info0013 b/t/t5100/info0013
new file mode 100644
index 0000000..bbe049e
--- /dev/null
+++ b/t/t5100/info0013
@@ -0,0 +1,5 @@
+Author: A U Thor
+Email: a.u.thor@example.com
+Subject: a patch
+Date: Fri, 9 Jun 2006 00:44:16 -0700
+
diff --git a/t/t5100/msg0013 b/t/t5100/msg0013
new file mode 100644
index 0000000..e69de29
diff --git a/t/t5100/patch0013 b/t/t5100/patch0013
new file mode 100644
index 0000000..e69de29
diff --git a/t/t5100/sample.mbox b/t/t5100/sample.mbox
index 4f80b82..c5ad206 100644
--- a/t/t5100/sample.mbox
+++ b/t/t5100/sample.mbox
@@ -556,3 +556,8 @@ index 3fd3afb..0ee807e 100644
#. п╫п╟ я│п╡п╬п╣п╧ п╪п╟я┬п╦п╫п╣: п╬я┌я─п╣п╢п╟п╨я┌п╦я─п╬п╡п╟я┌я▄ /etc/sudoers (п╨п╬п╪п╟п╫п╢п╟ ``visudo``) п©я─п╦п╪п╣я─п╫п╬ я│п╩п╣п╢я┐я▌я┴п╦п╪ п╬п╠я─п╟п╥п╬п╪::
diff --git a/symlinks.c b/symlinks.c
index 52b3b0e..f9d1821 100644
--- a/symlinks.c
+++ b/symlinks.c
@@ -1,10 +1,11 @@
#include "cache.h"
static struct cache_def {
- char path[PATH_MAX];
+ char path[PATH_MAX + 1];
int len;
int flags;
int track_flags;
+ int prefix_len_stat_func;
} cache;
/* Returns the length (on a path component basis) of the longest
@@ -30,12 +31,13 @@ static inline int greatest_match_lstat_cache(int len, const char *name)
return match_len;
}
-static inline void reset_lstat_cache(int track_flags)
+static inline void reset_lstat_cache(int track_flags, int prefix_len_stat_func)
{
cache.path[0] = '\0';
cache.len = 0;
cache.flags = 0;
cache.track_flags = track_flags;
+ cache.prefix_len_stat_func = prefix_len_stat_func;
}
#define FL_DIR (1 << 0)
@@ -43,28 +45,35 @@ static inline void reset_lstat_cache(int track_flags)
#define FL_SYMLINK (1 << 2)
#define FL_LSTATERR (1 << 3)
#define FL_ERR (1 << 4)
+#define FL_FULLPATH (1 << 5)
/*
* Check if name 'name' of length 'len' has a symlink leading
* component, or if the directory exists and is real, or not.
*
* To speed up the check, some information is allowed to be cached.
- * This is can be indicated by the 'track_flags' argument.
+ * This is can be indicated by the 'track_flags' argument, which also
+ * can be used to indicate that we should always check the full path.
+ *
+ * The 'prefix_len_stat_func' parameter can be used to set the length
+ * of the prefix, where the cache should use the stat() function
+ * instead of the lstat() function to test each path component.
*/
static int lstat_cache(int len, const char *name,
- int track_flags)
+ int track_flags, int prefix_len_stat_func)
{
int match_len, last_slash, last_slash_dir;
- int match_flags, ret_flags, save_flags, max_len;
+ int match_flags, ret_flags, save_flags, max_len, ret;
struct stat st;
- if (cache.track_flags != track_flags) {
+ if (cache.track_flags != track_flags ||
+ cache.prefix_len_stat_func != prefix_len_stat_func) {
/*
* As a safeguard we clear the cache if the value of
* track_flags does not match with the last supplied
* value.
*/
- reset_lstat_cache(track_flags);
+ reset_lstat_cache(track_flags, prefix_len_stat_func);
match_len = last_slash = 0;
} else {
/*
@@ -96,12 +105,17 @@ static int lstat_cache(int len, const char *name,
cache.path[match_len] = name[match_len];
match_len++;
} while (match_len < max_len && name[match_len] != '/');
- if (match_len >= max_len)
+ if (match_len >= max_len && !(track_flags & FL_FULLPATH))
break;
last_slash = match_len;
cache.path[last_slash] = '\0';
- if (lstat(cache.path, &st)) {
+ if (last_slash <= prefix_len_stat_func)
+ ret = stat(cache.path, &st);
+ else
+ ret = lstat(cache.path, &st);
+
+ if (ret) {
ret_flags = FL_LSTATERR;
if (errno == ENOENT)
ret_flags |= FL_NOENT;
@@ -121,12 +135,12 @@ static int lstat_cache(int len, const char *name,
* for the moment!
*/
save_flags = ret_flags & track_flags & (FL_NOENT|FL_SYMLINK);
- if (save_flags && last_slash > 0 && last_slash < PATH_MAX) {
+ if (save_flags && last_slash > 0 && last_slash <= PATH_MAX) {
cache.path[last_slash] = '\0';
cache.len = last_slash;
cache.flags = save_flags;
} else if (track_flags & FL_DIR &&
- last_slash_dir > 0 && last_slash_dir < PATH_MAX) {
+ last_slash_dir > 0 && last_slash_dir <= PATH_MAX) {
/* We have a separate test for the directory case,
* since it could be that we have found a symlink or a
* none existing directory and the track_flags says
@@ -141,17 +155,19 @@ static int lstat_cache(int len, const char *name,
cache.len = last_slash_dir;
cache.flags = FL_DIR;
} else {
- reset_lstat_cache(track_flags);
+ reset_lstat_cache(track_flags, prefix_len_stat_func);
}
return ret_flags;
}
+#define USE_ONLY_LSTAT 0
+
/* Return non-zero if path 'name' has a leading symlink component.
*/
int has_symlink_leading_path(int len, const char *name)
{
return lstat_cache(len, name,
- FL_SYMLINK|FL_DIR) &
+ FL_SYMLINK|FL_DIR, USE_ONLY_LSTAT) &
FL_SYMLINK;
}
@@ -161,6 +177,19 @@ int has_symlink_leading_path(int len, const char *name)
int has_symlink_or_noent_leading_path(int len, const char *name)
{
return lstat_cache(len, name,
- FL_SYMLINK|FL_NOENT|FL_DIR) &
+ FL_SYMLINK|FL_NOENT|FL_DIR, USE_ONLY_LSTAT) &
(FL_SYMLINK|FL_NOENT);
}
+
+/* Return non-zero if all path components of 'name' exists as a
+ * directory. If prefix_len > 0, we will test with the stat()
+ * function instead of the lstat() function for a prefix length of
+ * 'prefix_len', thus we then allow for symlink(s) in the prefix part
+ * as long as those points to real existing directories.
+ */
+int has_dirs_only_path(int len, const char *name, int prefix_len)
+{
+ return lstat_cache(len, name,
+ FL_DIR|FL_FULLPATH, prefix_len) &
+ FL_DIR;
+}
@@ -43,28 +45,35 @@ static inline void reset_lstat_cache(int track_flags)
/*
- * As a safeguard we clear the cache if the value of
- * track_flags does not match with the last supplied
- * value.
+ * As a safeguard we clear the cache if the values of
+ * track_flags and/or prefix_len_stat_func does not
+ * match with the last supplied values.
*/
diff --git a/symlinks.c b/symlinks.c
index b17e4b3..52b3b0e 100644
--- a/symlinks.c
+++ b/symlinks.c
@@ -4,6 +4,7 @@ static struct cache_def {
char path[PATH_MAX];
int len;
int flags;
+ int track_flags;
} cache;
/* Returns the length (on a path component basis) of the longest
@@ -29,21 +30,23 @@ static inline int greatest_match_lstat_cache(int len, const char *name)
return match_len;
}
-static inline void reset_lstat_cache(void)
+static inline void reset_lstat_cache(int track_flags)
{
cache.path[0] = '\0';
cache.len = 0;
cache.flags = 0;
+ cache.track_flags = track_flags;
}
#define FL_DIR (1 << 0)
-#define FL_SYMLINK (1 << 1)
-#define FL_LSTATERR (1 << 2)
-#define FL_ERR (1 << 3)
+#define FL_NOENT (1 << 1)
+#define FL_SYMLINK (1 << 2)
+#define FL_LSTATERR (1 << 3)
+#define FL_ERR (1 << 4)
/*
* Check if name 'name' of length 'len' has a symlink leading
- * component, or if the directory exists and is real.
+ * component, or if the directory exists and is real, or not.
*
* To speed up the check, some information is allowed to be cached.
* This is can be indicated by the 'track_flags' argument.
@@ -55,22 +58,32 @@ static int lstat_cache(int len, const char *name,
int match_flags, ret_flags, save_flags, max_len;
struct stat st;
- /*
- * Check to see if we have a match from the cache for the
- * symlink path type.
- */
- match_len = last_slash = greatest_match_lstat_cache(len, name);
- match_flags = cache.flags & track_flags & FL_SYMLINK;
- if (match_flags && match_len == cache.len)
- return match_flags;
- /*
- * If 'name' is a substring of the cache on a path component
- * basis, and a directory is cached, we can return
- * immediately.
- */
- match_flags = cache.flags & track_flags & FL_DIR;
- if (match_flags && match_len == len)
- return match_flags;
+ if (cache.track_flags != track_flags) {
+ /*
+ * As a safeguard we clear the cache if the value of
+ * track_flags does not match with the last supplied
+ * value.
+ */
+ reset_lstat_cache(track_flags);
+ match_len = last_slash = 0;
+ } else {
+ /*
+ * Check to see if we have a match from the cache for
+ * the 2 "excluding" path types.
+ */
+ match_len = last_slash = greatest_match_lstat_cache(len, name);
+ match_flags = cache.flags & track_flags & (FL_NOENT|FL_SYMLINK);
+ if (match_flags && match_len == cache.len)
+ return match_flags;
+ /*
+ * If 'name' is a substring of the cache on a path
+ * component basis, and a directory is cached, we
+ * can return immediately.
+ */
+ match_flags = cache.flags & track_flags & FL_DIR;
+ if (match_flags && match_len == len)
+ return match_flags;
+ }
/* Okay, no match from the cache so far, so now we have to
* check the rest of the path components.
@@ -90,6 +103,8 @@ static int lstat_cache(int len, const char *name,
if (lstat(cache.path, &st)) {
ret_flags = FL_LSTATERR;
+ if (errno == ENOENT)
+ ret_flags |= FL_NOENT;
} else if (S_ISDIR(st.st_mode)) {
last_slash_dir = last_slash;
continue;
@@ -101,11 +116,11 @@ static int lstat_cache(int len, const char *name,
break;
}
/* At the end update the cache. Note that max 2 different
* path types, FL_SYMLINK and FL_DIR, can be cached for the
* moment!
*/
- save_flags = ret_flags & track_flags & FL_SYMLINK;
+ save_flags = ret_flags & track_flags & (FL_NOENT|FL_SYMLINK);
if (save_flags && last_slash > 0 && last_slash < PATH_MAX) {
cache.path[last_slash] = '\0';
cache.len = last_slash;
@@ -113,20 +128,20 @@ static int lstat_cache(int len, const char *name,
} else if (track_flags & FL_DIR &&
last_slash_dir > 0 && last_slash_dir < PATH_MAX) {
/* We have a separate test for the directory case,
- * since it could be that we have found a symlink and
- * the track_flags says that we can not cache this
- * fact, so the cache would then have been left empty
- * in this case.
+ * since it could be that we have found a symlink or a
+ * none existing directory and the track_flags says
+ * that we can not cache this fact, so the cache would
+ * then have been left empty in this case.
*
* But, if we is allowed to track real directories, we
* can still cache the path components before the last
- * one (the found symlink component).
+ * one (the found symlink or none existing component).
*/
cache.path[last_slash_dir] = '\0';
cache.len = last_slash_dir;
cache.flags = FL_DIR;
} else {
- reset_lstat_cache();
+ reset_lstat_cache(track_flags);
}
return ret_flags;
}
@@ -139,3 +154,13 @@ int has_symlink_leading_path(int len, const char *name)
FL_SYMLINK|FL_DIR) &
FL_SYMLINK;
}
+
+/* Return non-zero if path 'name' has a leading symlink component or
+ * if some leading path component does not exists.
+ */
+int has_symlink_or_noent_leading_path(int len, const char *name)
+{
+ return lstat_cache(len, name,
+ FL_SYMLINK|FL_NOENT|FL_DIR) &
+ (FL_SYMLINK|FL_NOENT);
+}
@@ -101,11 +116,11 @@ static int lstat_cache(int len, const char *name,
break;
}
- /* At the end update the cache. Note that max 2 different
- * path types, FL_SYMLINK and FL_DIR, can be cached for the
- * moment!
+ /* At the end update the cache. Note that max 3 different
+ * path types, FL_NOENT, FL_SYMLINK and FL_DIR, can be cached
+ * for the moment!
*/
--
1.5.6.5
+From nobody Mon Sep 17 00:00:00 2001
+From: <a.u.thor@example.com> (A U Thor)
+Date: Fri, 9 Jun 2006 00:44:16 -0700
+Subject: [PATCH] a patch
+
--
1.6.1.101.g0335
^ permalink raw reply related
* Re: [PATCH] Teach format-patch to handle output directory relative to cwd
From: Cesar Eduardo Barros @ 2009-01-13 9:24 UTC (permalink / raw)
To: Junio C Hamano; +Cc: git
In-Reply-To: <7vmydwf2a9.fsf@gitster.siamese.dyndns.org>
Junio C Hamano escreveu:
> Cesar Eduardo Barros <cesarb@cesarb.net> writes:
>
>> Works great, only the resulting output to the screen is a bit
>> ugly/confusing:
>>
>> drivers/net/../../../0001-sc92031-more-useful-banner-in-kernel-log.patch
>> ...
>> (after all, I am still inside the drivers/net directory)
>
> Fair enough. Here is a replacement diff.
Works fine, thanks.
--
Cesar Eduardo Barros
cesarb@cesarb.net
cesar.barros@gmail.com
^ permalink raw reply
* Re: [PATCH] Fix Documentation typos surrounding the word 'handful'.
From: Junio C Hamano @ 2009-01-13 9:24 UTC (permalink / raw)
To: Jon Loeliger; +Cc: git
In-Reply-To: <1231790527-7515-1-git-send-email-jdl@freescale.com>
Thanks, applied with fix from Markus and Sam.
^ permalink raw reply
* Re: [PATCH next] notes: fix core.notesRef documentation
From: Junio C Hamano @ 2009-01-13 9:24 UTC (permalink / raw)
To: Johannes Schindelin; +Cc: Thomas Rast, git
In-Reply-To: <alpine.DEB.1.00.0901121541210.3586@pacific.mpi-cbg.de>
Thanks, queued.
^ permalink raw reply
* Re: How to pull remote branch with specified commit id?
From: Andreas Ericsson @ 2009-01-13 9:16 UTC (permalink / raw)
To: Emily Ren; +Cc: Git Mailinglist
In-Reply-To: <856bfe0e0901130108q3af1345cy31751dd09e030c96@mail.gmail.com>
Emily Ren wrote:
> Git experts,
>
> I want to pull remote branch with specified commit id, how to do it?
>
> Below command can get remote branch
> $git pull remote refs/heads/$branch_name
>
> Below command doesn't work
> $git pull remote objects/$commit_id
>
You need to fetch it first, and then merge the commit you want. The
tools operating the fetching protocol only use refs, so if you want
to fetch (or pull) a specific version that has neither a tag nor a
branch head pointing to it, you'll have to write a new tool for that.
The end-result of the following command will be, barring side-effects
in the remote-tracking branches, identical to what you're trying to
do though:
git fetch remote && git merge $commit_id
--
Andreas Ericsson andreas.ericsson@op5.se
OP5 AB www.op5.se
Tel: +46 8-230225 Fax: +46 8-230231
^ permalink raw reply
* How to pull remote branch with specified commit id?
From: Emily Ren @ 2009-01-13 9:08 UTC (permalink / raw)
To: Git Mailinglist
Git experts,
I want to pull remote branch with specified commit id, how to do it?
Below command can get remote branch
$git pull remote refs/heads/$branch_name
Below command doesn't work
$git pull remote objects/$commit_id
Thanks,
Emily
^ permalink raw reply
page: next (older) | prev (newer) | latest
- recent:[subjects (threaded)|topics (new)|topics (active)]
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox